require 'pcaprub'
require 'uri'

# helper module for pcaprub
module PcaprubHelper
  class << self
    attr_reader :cap, :last_cap
    attr_accessor :interface, :snaplen, :promisc, :timeout
    attr_accessor :filter, :url_filter

    def configure
      yield self
    end

    def capture_this
      build_capture
      yield
      @last_cap = parse_capture
      destroy_capture
    end

    def verify_query_string(**kwargs)
      return false unless @last_cap

      @last_cap.each do |packet|
        kwargs.each do |key, value|
          return false unless packet =~ query_string_validator(key, value)
        end
      end

      true
    end

    def verify_path_string(*args)
      return false unless @last_cap

      @last_cap.each do |packet|
        args.each do |value|
          return false unless packet =~ path_string_validator(value)
        end
      end

      true
    end

    def look_up_device
      PCAPRUB::Pcap.lookupdev
    rescue PCAPRUB::BindingError
      puts "lookupdev : no suitable device found, defaulting to 'en0'"
      'en0'
    end

    private

    def query_string_validator(key, value)
      /(\&|\?)#{key}[\[\]]{0,2}=#{value}(\&|\s)/i
    end

    def path_string_validator(value)
      %r{\/#{value}\/i}
    end

    def build_capture
      @cap = PCAPRUB::Pcap.open_live(@interface, @snaplen, @promisc, @timeout)
      @cap.setfilter(@filter || '')
    end

    def destroy_capture
      @cap.close
      @cap = nil
    end

    def parse_capture
      packets = []
      loop do
        packet = @cap.next
        break unless packet
        next unless filter(packet)
        packets << URI.decode(packet)
      end
      packets
    end

    def filter(packet)
      @url_filter ? packet =~ @url_filter : true
    end
  end
end

PcaprubHelper.configure do |config|
  config.interface = config.look_up_device
  config.snaplen   = 65_535
  config.promisc   = true
  config.timeout   = 0
end
